from units import *
from orbit import *
from math import *

radian = 1.0
degree = pi / 180
twopi = 2 * pi

armLength = 800 * mile
omega = circularOmegaFromRadius(earthRadius + armLength)
cogSpeed = circularVelocityFromRadius(earthRadius + armLength)
groundSpeed = omega * earthRadius
spinRate = groundSpeed / armLength
spinPeriod = twopi / spinRate

print armLength.inTermsOf(mile), "length of pinwheel arms"

print groundSpeed.inTermsOf(MPH), "ground speed of pinwheel"

# assume a max acceptable vertical range, then back-compute the
# capture time

verticalPlay = 100 * foot
sliver = 2 * acos(1 - verticalPlay / armLength)
window = sliver / spinRate

print window.inTermsOf(second), "to grab the cable"

print (pi / spinRate).inTermsOf(minute), "riding the pinwheel"

print (2 * cogSpeed).inTermsOf(MPS), "speed when leaving the pinwheel"

earthCircumference = twopi * earthRadius
hop = groundSpeed * spinPeriod
#print hop.inTermsOf(mile), "hop"
n = int(earthCircumference / hop + 0.5)
print n, "pinwheel rotations til next try"
print (n * spinPeriod).inTermsOf(minute), "til next try"
print abs(n * hop - earthCircumference).inTermsOf(mile), "to next try"

#o = Orbit(earthRadius + 2 * armLength, 2 * cogSpeed)
#print o.perigee
#print o.apogee



from net.willware.Orbits import System, Vector, PointMass, Tether
from java.awt import Color
import sys

incomingStartingRadius = System.earthRadius + 3000 * System.mile
outgoingStartingRadius = System.earthRadius + 20 * System.mile
incomingStartingVelocity = 14100

s = System()
LET_GO_TIME = 1530
s.tmax = 5000
s.dt = 0.1
s.time = 0.0

def getarg(args=sys.argv[1:]):
    return args.pop(0)

while 1:
    try:
        arg = getarg()
        if arg == "-tmax":
            s.tmax = eval(getarg())
        elif arg == "-letgo":
            LET_GO_TIME = eval(getarg())
    except IndexError:
        break

incomingShip = PointMass(s, "IncomingShip", System.airplaneMass,
                         Vector(0, incomingStartingRadius, 0),
                         Vector(incomingStartingVelocity, 0, 0),
                         Color.MAGENTA)

vlow = System.airplaneVelocity + System.earthRotationalVelocity
outgoingShip = PointMass(s, "OutgoingShip", System.airplaneMass,
                         Vector(0, outgoingStartingRadius, 0),
                         Vector(-vlow, 0, 0),
                         Color.ORANGE);

tether = Tether(incomingStartingRadius - outgoingStartingRadius,
                LET_GO_TIME)
tether.setEnd1(incomingShip)
tether.setEnd2(outgoingShip)
s.add(incomingShip)
s.add(outgoingShip)
s.add(tether)
s.hack()


print "\nOutgoing ship:"
print ("min altitude %g miles" % (outgoingShip.minAltitude / System.mile))
print ("max altitude %g miles" % (outgoingShip.maxAltitude / System.mile))
print ("min altitude %g km" % (outgoingShip.minAltitude / 1000))
print ("max altitude %g km" % (outgoingShip.maxAltitude / 1000))
print ("max velocity %g m/sec" % outgoingShip.maxVelocity)
print ("final velocity %g m/sec" % (outgoingShip.previousVelocity))
print ("max acceleration %g Gs" %
       (outgoingShip.maxAcceleration / System.earthGravity))

print "\nIncoming ship:"
print ("init velocity %g m/sec" % incomingStartingVelocity)
print ("min velocity %g m/sec" % incomingShip.minVelocity)
print ("max velocity %g m/sec" % incomingShip.maxVelocity)

print "\nTether:"
print ("max tension %g newtons" % tether.getMaxTension())
print ("stretch factor %g" % (tether.getMaxLength() / tether.getLength()))



"""Design the taper for a space-dock-style tower. Assume synthetic
diamond with reasonable material properties. No unobtainium."""

import units

density = 3500 * units.kg / units.m**3
# This is the actually achievable compressive strength of
# synthetic diamond.
compressiveStrength = 5.e9 * units.pascal
# To build conservatively with a large safety margin, let's
# assume a much lower compressive strength.
compressiveStrength = 0.1 * compressiveStrength

topMass = 1000 * units.metricTon
topAltitude = 100 * units.km

def gravity(altitude):
    # inverse square law, returns an acceleration
    R = units.earthRadius
    ratio = (R / (R + altitude)) ** 2
    return ratio * units.earthGravity

h = topAltitude
force = topMass * gravity(topAltitude)
dh = 0.1 * units.km
volume = 0. * units.m**3

# For each dh step in altitude, design a cylinder to hold up the
# weight above, then add its weight to the weight of the weight of
# the stuff above it. Start at the top of the tower and iterate
# down to the ground.

crossSections = [ ]

while h > -units.cm:
    # support the weight above this piece
    area = force / compressiveStrength
    crossSections.append((h, area))
    h = h - dh
    # how much weight does this piece add?
    stepVolume = dh * area
    volume = volume + stepVolume
    force = force + stepVolume * density * gravity(h)

print "All these quantities scale linearly with topMass"
print "topMass", topMass
print "Cross section at the top", crossSections[0]
print "Cross section at the bottom", crossSections[-1]
print "Load resting on the ground", force / units.earthGravity
print "Volume of synthetic diamond", volume
print "Mass of synthetic diamond", volume * density



from net.willware.Orbits import System, Vector, PointMass, Tether
from net.willware.Orbits.System import *
from java.awt import Color
import sys

incomingStartingRadius = earthRadius + 3000 * mile
outgoingStartingRadius = earthRadius + 20 * mile
incomingStartingVelocity = 14100

s = System()
LET_GO_TIME = 1.0e20
s.tmax = 3 * hour
s.dt = 0.1
s.time = 0.0

def getarg(args=sys.argv[1:]):
    return args.pop(0)

while 1:
    try:
        arg = getarg()
        if arg == "-tmax":
            s.tmax = eval(getarg())
        elif arg == "-letgo":
            LET_GO_TIME = eval(getarg())
    except IndexError:
        break

pinwheelMassRatio = 3.0
pinwheelRadius = earthRadius + 2000 * mile

lowpoint = earthRadius + 30 * mile
# 1.025 for mass ratio of 50
# 1.113 for mass ratio of 10
# 1.41 for mass ratio of 3
pinwheelVelocity = 1.4105 * circularVelocityFromRadius(pinwheelRadius)

if 1:
    totalPinMass = airplaneMass * (1 + pinwheelMassRatio)
    a = pinwheelMassRatio
    pinCog = (a * pinwheelRadius + lowpoint) / (a + 1)
    expected = ((a + 1) / a) * circularVelocityFromRadius(pinCog)
    print pinwheelVelocity / expected
    pinwheelVelocity = 1.0132 * expected

lowpointVelocity = earthRotationalVelocity

pinwheelCenter = PointMass(s, "Pinwheel",
                           pinwheelMassRatio * airplaneMass,
                           Vector(0, pinwheelRadius, 0),
                           Vector(pinwheelVelocity, 0, 0),
                           Color.MAGENTA)
ship = PointMass(s, "Ship", airplaneMass,
                 Vector(0, lowpoint, 0),
                 Vector(-lowpointVelocity, 0, 0),
                 Color.ORANGE)
tether = Tether(pinwheelRadius - lowpoint, LET_GO_TIME)
tether.setEnd1(pinwheelCenter)
tether.setEnd2(ship)
s.add(pinwheelCenter)
s.add(ship)
s.add(tether)
s.hack()

print "\nOutgoing ship:"
print ("min altitude %g miles" % (ship.minAltitude / mile))
print ("max altitude %g miles" % (ship.maxAltitude / mile))
print ("min altitude %g km" % (ship.minAltitude / 1000))
print ("max altitude %g km" % (ship.maxAltitude / 1000))
print ("max velocity %g m/sec" % ship.maxVelocity)
print ("final velocity %g m/sec" % (ship.previousVelocity))
print ("max acceleration %g Gs" %
       (ship.maxAcceleration / earthGravity))

print "\nPinwheel:"
print ("min altitude %g miles" % (pinwheelCenter.minAltitude / mile))
print ("max altitude %g miles" % (pinwheelCenter.maxAltitude / mile))
print ("min altitude %g km" % (pinwheelCenter.minAltitude / 1000))
print ("max altitude %g km" % (pinwheelCenter.maxAltitude / 1000))
print ("min velocity %g m/sec" % pinwheelCenter.minVelocity)
print ("max velocity %g m/sec" % pinwheelCenter.maxVelocity)

print "\nTether:"
print ("max tension %g newtons" % tether.getMaxTension())



import units, math

GM = units.gravitationalConstant * units.earthMass

class EscapeVelocity(Exception):
    pass

class CrashIntoEarth(Exception):
    pass

class Orbit:
    # http://scienceworld.wolfram.com/physics/Two-BodyProblem.html

    # Supply (r,v) for either perigee or apogee
    def __init__(self, r1, v1):
        rv = r1 * v1
        # energy per kilogram
        E = 0.5 * v1**2 - GM / r1
        if E.coeff >= 0:
            raise EscapeVelocity
        # Energy per kilogram, in excess of the mass just sitting
        # on the surface of the Earth
        self.E = E + (GM / units.earthRadius)
        a, b, c = E, GM, -0.5 * rv**2
        d = (b**2 - 4 * a * c)**0.5
        x1 = (-b - d) / (2 * a)
        x2 = (-b + d) / (2 * a)
        if x2 < x1: x1, x2 = x2, x1
        self.perigee = (x1, rv / x1)
        self.apogee = (x2, rv / x2)
        safeAltitude = units.earthRadius + 150 * units.km
        if self.perigee[0] < safeAltitude:
            raise CrashIntoEarth

##################

def circularVelocityFromRadius(r):
    return (GM / r) ** .5

def circularRadiusFromVelocity(v):
    return GM / v**2

def circularRadiusFromOmega(w):
    return (GM / w**2) ** (1./3)

def circularOmegaFromRadius(r):
    return (GM / r**3) ** .5

def escapeVelocityFromRadius(r):
    return (2 * GM / r) ** .5

# I picked up a 12-inch globe, and I want to show where
# geosynchronous orbit is.
def geo():
    geosync = circularRadiusFromOmega(2 * math.pi / (24 * units.hour))
    print "geosynchronous orbit at", geosync
    print "or in miles", geosync.inTermsOf(units.mile)
    geosync -= units.earthRadius
    print "geosynchronous orbit above ground", geosync
    print "or in miles", geosync.inTermsOf(units.mile)

    print "geosync altitude in earth diameters",
    print geosync / (2 * units.earthRadius)

    globeScale = units.earthRadius / (6 * units.inch)
    print "geosync on 12-inch globe",
    print (geosync / globeScale).inTermsOf(units.inch)

def apogeeFromPerigee(rmin, vmin):
    return Orbit(rmin, vmin).apogee

def main():
    for i in range(1,7):
        dv = i * 500 * units.m / units.sec

        ###############################################################
        # We start out launching from JoSH's space ramp, going faster than
        # we need for a circular orbit, this puts us in an elliptic orbit
        # starting at the end of the space ramp. See
        # http://www.imm.org/Reports/Rep016.html.

        rmin = units.earthRadius + 100 * units.km
        vmin = circularVelocityFromRadius(rmin) + dv
        rmax, vmax = apogeeFromPerigee(rmin, vmin)

        print "dv", dv
        print "perigee", vmin, rmin
        print "apogee", vmax, rmax

        ################################################################
        # When we reach apogee, we grab the bottom end of a space elevator
        # cable. The space elevator is a rope with a weight in the middle,
        # and the weight is in a circular orbit. Because this elevator is
        # not in geosynchronous orbit, it doesn't need to be as long or
        # have the tensile strength of a geosynchronous space elevator.
        # Our velocity at apogee tells us the orbital radius of the
        # weight.

        omega = vmax / rmax
        rweight = circularRadiusFromOmega(omega)
        print "orbital period for weight", \
              (2 * math.pi / omega) / units.hour, "hours"

        ################################################################
        # We climb the cable to the counterweight. We pay the station
        # operator in rocket fuel, which he uses to correct the orbit of
        # the elevator which we've perturbed. We continue upward to the
        # top end of the cable. The upper end of the cable can be as long
        # as we want -- it's not subject to much tension.

        # Cable should be much longer above than below?
        rtop = rweight + 3 * (rweight - rmax)

        print "Lower cable length", rweight - rmax
        print "Altitude of weight", rweight
        print "Upper cable length", rtop - rweight
        print "Total cable length", rtop - rmax

        ################################################################
        # Now we let go of the upper end of the cable. Big question: do
        # we attain escape velocity for Earth? Tidal forces tend to keep
        # the cable radially oriented to the Earth, and maybe if we climb
        # the upper cable slowly, we won't mess that up. If so, then we
        # keep the same angular velocity, and linear velocity scales with
        # radius.

        print "Escape velocity?", escapeVelocityFromRadius(rtop), "<", \
              vmax * (rtop / rmax)
        print



'''units.py - physical units for calculations

When quantities are multiplied or divided, the powers of units are
correctly maintained. If one unit ends up being raised to a zeroth
power, that unit is discarded in the product or ratio. If there are
no units left at all, i.e. the result is dimensionless, just return
a float.

Many physical formulas apply transcendental functions to dimensionless
ratios of quantities. For instance, the voltage across a discharging
capacitor is v0*exp(-t/(R*C)), where t, R, and C have dimensions
(seconds, ohms, and farads), but the ratio is dimensionless. So
returning a float in such cases is a desirable behavior.

There are also some handy units and conversions at the bottom of the
file.'''

import types

def isnumber(x):
    if type(x) in (types.IntType, types.FloatType, types.ComplexType):
        return 1
    return 0

class UnitsMismatch(Exception):
    pass

class Quantity:
    def __init__(self,coeff,units,name=None):
        self.coeff = 1. * coeff
        self.units = units.copy()
        if name != None:
            self.name = name
    def __repr__(self):
	str = '<'
        try:
            str = str + self.name + ' '
        except AttributeError:
            pass
        try: str = str + '%g' % self.coeff
        except TypeError: str = str + repr(self.coeff)
	for k in self.units.keys():
            str = str + ' ' + k
            if self.units[k] != 1:
                str = str + '^' + repr(self.units[k])
	return str + '>'

#   def __abs__(self):
#	return Quantity(abs(self.coeff), self.units.copy())

    def __abs__(self):
	return abs(self.coeff)

    def __neg__(self):
	return Quantity(-self.coeff, self.units.copy())

    def __cmp__(self,other):
	self.unitsMatch(other)
	return cmp(self.coeff, other.coeff)

    def __add__(self,other):
	self.unitsMatch(other)
	units = self.units.copy()
	return Quantity(self.coeff + other.coeff,
			units)
    def __sub__(self,other):
	self.unitsMatch(other)
	units = self.units.copy()
	return Quantity(self.coeff - other.coeff,
			units)
    def __mul__(self,other):
        if isinstance(other, Quantity):
            c = 1. * self.coeff * other.coeff
            u = self.multUnits(other, 1)
            if u.keys() == [ ]:
                return c
            else:
                return Quantity(c, u)
        elif isnumber(other):
            return Quantity(other * self.coeff, self.units)
        else:
            raise RuntimeError
    def __rmul__(self, other):
        return self.__mul__(other)
    def __div__(self,other):
        if isinstance(other, Quantity):
            return self * other.reciprocal()
        return Quantity(self.coeff / other, self.units)
    def __rdiv__(self, other):
        return other * self.reciprocal()
    def __pow__(self, n):
        if n == 0:
            return 1.
        if n < 0:
            return 1. / self.__pow__(-n)
        newcoeff = self.coeff ** n
        newunits = { }
        for k in self.units.keys():
            newunits[k] = self.units[k] * n
        return Quantity(newcoeff, newunits)
    def reciprocal(self):
        a = 1. / self.coeff
	b = self.units.copy()
	for k in b.keys():
	    b[k] = -b[k]
        return Quantity(a, b)
    def inTermsOf(self, other):
        ratio = self / other
        try: str = "%g" % ratio
        except TypeError: str = repr(ratio)
        return str + " " + other.name + "s"
    def multUnits(self,other,n):
	units1 = self.units.copy()
	units2 = other.units.copy()
	for k in units1.keys():
	    if k in units2.keys():
		units1[k] = units1[k] + n * units2[k]
	for k in units2.keys():
	    if k not in units1.keys():
		units1[k] = n * units2[k]
        for k in units1.keys():
            if units1[k] == 0:
                del units1[k]
	return units1
    def unitsMatch(self,other):
	otherkeys = other.units.keys()
	for k in self.units.keys():
	    if k not in otherkeys:
		raise UnitsMismatch
	    if k != "coeff" and self.units[k] != other.units[k]:
		raise UnitsMismatch

# Lotsa good stuff on units and measures at:
# http://aurora.rg.iupui.edu/UCUM/UCUM-tab.html

### Fundamental units
meter = m = Quantity(1,{'m':1},"meter")
kilogram = kg = Quantity(1,{'kg':1},"kilogram")
second = sec = Quantity(1,{'sec':1},"second")
coulomb = C = Quantity(1,{'C':1},"coulomb")
radian = rad = Quantity(1,{'rad':1},"radian")
steradian = srad = Quantity(1,{'sr':1},"steradian")

Deca = 10.0
Hecto = 100.0
Kilo = 1.e3
Mega = 1.e6
Giga = 1.e9
Tera = 1.e12
Peta = 1.e15
Exa = 1.e18
Zetta = 1.e21
Yotta = 1.e24

Deci = 0.1
Centi = 0.01
Milli = 1.e-3
Micro = 1.e-6
Nano = 1.e-9
Pico = 1.e-12
Femto = 1.e-15
Atto = 1.e-18
Zepto = 1.e-21
Yocto = 1.e-24

def _make(unit,name):
    unit.name = name
    return unit

### Conveniences and metric prefixes
squareMeter = _make(meter ** 2, "squareMeter")
cubicMeter = _make(meter ** 3, "cubicMeter")

# don't know the official metric names for these, if such exist
speed = meter / second
acceleration = speed / second
density = kilogram / cubicMeter

liter = L = _make(Milli * cubicMeter, "liter")
newton = N = _make(kilogram * acceleration, "newton")
joule = J = _make(newton * meter, "joule")
watt = W = _make(joule / second, "watt")
ampere = A = _make(coulomb / second, "ampere")
volt = V = _make(joule / coulomb, "volt")
ohm = _make(volt / ampere, "ohm")
farad = F = _make(coulomb / volt, "farad")
weber = Wb = _make(volt * second, "weber")
tesla = T = _make(weber / squareMeter, "tesla")
pascal = P = _make(newton / squareMeter, "pascal")
henry = H = _make(weber / ampere, "henry")

calorie = cal = _make(4.1868 * joule, "calorie")
kilocalorie = kcal = _make(Kilo * calorie, "kcal")

km = _make(Kilo * meter, "km")
cm = _make(Centi * meter, "cm")
mm = _make(Milli * meter, "mm")
um = _make(Micro * meter, "um")
nm = _make(Nano * meter, "nm")

are = _make(100 * squareMeter, "are")
hectare = _make(Hecto * are, "hectare")

gram = g = _make(Milli * kilogram, "gram")
mgram = mg = _make(Micro * kilogram, "mgram")
ugram = ug = _make(Nano * kilogram, "ugram")
metricTon = tonne = _make(Kilo * kilogram, "tonne")

A = _make(ampere, "A")
mA = _make(Milli * ampere, "mA")
uA = _make(Micro * ampere, "uA")
nA = _make(Nano * ampere, "nA")
pA = _make(Pico * ampere, "pA")

msec = _make(Milli * second, "msec")
usec = _make(Micro * second, "usec")
nsec = _make(Nano * second, "nsec")
psec = _make(Pico * second, "psec")
fsec = _make(Femto * second, "fsec")

ksec = _make(Kilo * second, "ksec")
Msec = _make(Mega * second, "Msec")
Gsec = _make(Giga * second, "Gsec")
Tsec = _make(Tera * second, "Tsec")

uF = _make(Micro * farad, "uF")
nF = _make(Nano * farad, "nF")
pF = _make(Pico * farad, "pF")

mH = _make(Milli * henry, "mH")
uH = _make(Micro * henry, "uH")
nH = _make(Nano * henry, "nH")

Kohm = _make(Kilo * ohm, "Kohm")
Mohm = _make(Mega * ohm, "Mohm")

# earthGravity = 9.8 * acceleration

gravitationalConstant = 6.67300e-11 * m**3 * kg**-1 * sec**-2
earthMass = 5.9742e24 * kg
earthRadius = 6378.1 * km
earthGravity = gravitationalConstant * earthMass / earthRadius**2

### Electrostatics
electronCharge = 1.60217733e-19 * coulomb
electrostaticConstant = 8.98755e9 * (N * m**2) / (coulomb**2)

### Time
minute = _make(60 * second, "minute")
hour = _make(60 * minute, "hour")
day = _make(24 * hour, "day")
week = _make(7 * day, "week")
year = _make(365.25 * day, "year")
# There is some disagreement on years. These are Julian years.
# Gregorian years are 365.2425 days, and tropical years are 365.24219
# days.
month = _make(year / 12, "month")
# 28, 29, 30, 31... obviously months must be approximate
decade = _make(10 * year, "decade")
century = _make(100 * year, "century")
millenium = _make(1000 * year, "millenium")

### English measures
inch = _make(.0254 * meter, "inch")
foot = _make(12 * inch, "foot")
yard = _make(3 * foot, "yard")
furlong = _make(660 * foot, "furlong")
mile = _make(5280 * foot, "mile")
squareMile = _make(mile ** 2, "squareMile")
acre = _make(43560 * foot**2, "acre")

MPH = _make(mile / hour, "MPH")
MPS = _make(mile / second, "MPS")

gallon = _make(3.79 * liter, "gallon")
quart = _make(.25 * gallon, "quart")
pint = _make(.5 * quart, "pint")
cup = _make(.5 * pint, "cup")
fluid_ounce = _make(.125 * cup, "fluid_ounce")
tablespoon = _make(.5 * fluid_ounce, "tablespoon")
teaspoon = _make(tablespoon / 3., "teaspoon")

# In the English system, should we express weights in terms of
# masses? Or should we multiply masses by 32 feet/sec^2?
ounce = oz = earthGravity * (28.35 * gram)
pound = lb = 16 * ounce
slug = 14.5939 * kg
stone = 6.3503 * kg

if __name__ == "__main__":

    def test1():
        lists = [ [ "Length",
                    inch,
                    furlong,
                    yard,
                    mile,
                    km
                    ],
                  [ "Area",
                    squareMeter,
                    acre,
                    hectare,
                    squareMile,
                    ],
                  [ "Volume",
                    gallon,
                    pint,
                    liter,
                    fluid_ounce,
                    tablespoon,
                    cubicMeter
                    ],
                  [ "Time",
                    day,
                    week,
                    decade,
                    century,
                    ksec,
                    Msec,
                    Gsec
                    ],
                  [ "Random",
                    newton,
                    volt,
                    ampere
                    ]
                  ]
        for lst in lists:
            print "----- " + lst.pop(0) + " -----"
            for i in range(len(lst)):
                for j in range(len(lst)):
                    if i != j:
                        print "one %s is %s" % (lst[i].name,
                                                lst[i].inTermsOf(lst[j]))
            print

    def test2():
        charge = mA * hour
        time = 1390.0 * second
        print charge / time

    test1()
